;+
; NAME:
;
;   plot_density_laws
;
; PURPOSE:
;   Uses equatorial density profiles combined with previous analysis to create a power law fit to the data.
;
; CALLING SEQUENCE:
;
;   plot_density_laws
;
; INPUTS: Inputs files are hard coded into the program

;
; OUTPUTS:
;   Plots of density power laws produced from the viper anlysis
;
; PROCEDURE:
;   Unpacks information about densities at the centrifugal equator and uses a linear fit to the data to produce a power law curve.
;
; EXAMPLE:
;
;
; Created by Kaleb Bodisch


pro plot_density_laws

  for loop = 0, 8 do begin
    ; In csv file land
    ;
    ;
    !Except = 0 ; Prevent error messages from the Nans in files

    ; Inbound + Outbound (both V1 and V2) for R_SC GE 6 Rj (-INF - 64:08:51::10 FOR V1) and (-INF - INF FOR V2)
    v1i  = read_csv('V1_JUP_M_No_Out_In_CT.csv') ; This one had been overwritten on accident so different naming scheme slightly
    v2i  = read_csv('V2_JUP_M_Dec20_Isotropic_CEq.csv')
    lims = [6d,15.4d] ; Cutoff of 15.4 Rj for power law fitting
    XRANG = [4d,50d]
    LT6 = 0





    ;4, 5, 6, 7, 8, 9, 10, 11
    ;15, ....
    ;e, o+, o++, s+++, s++, s+, h+, o+hot, Na

    A = [16,16,32,32,32,1,23,16]
    Z = [1,2,1,2,3,1,1,1]
    CSV_Data_total = READ_CSV("V1_Data_CSV.csv", HEADER = CSVHeader)
    total_charge = CSV_Data_total.field12

    CSV_Data = READ_CSV("Ce_Data.csv", HEADER = CSVHeader)
    rho_sc = csv_data.field01*cos(csv_data.field02*!pi/180.)
    rho_ce = csv_data.field12*cos(csv_data.field13*!pi/180.)
    oplus = csv_data.field04
    oplusce = csv_data.field15
    odouble = csv_data.field05
    odoublece = csv_data.field16
    splus = csv_data.field06
    splusce = csv_data.field17
    sdouble = csv_data.field07
    sdoublece = csv_data.field18
    strip = csv_data.field08
    stripce = csv_data.field19
    H = csv_data.field09
    Hce = csv_data.field20
    Hot = csv_data.field11
    Hotce = csv_data.field22
    Na = csv_data.field10
    Nace = csv_data.field21
    Total_ce = csv_data.field23






    IF loop eq 0 THEN BEGIN
      var1i = DOUBLE(v1i.Field52)
      var2i = DOUBLE(v2i.Field52)
      dn = total_charge
      dnce = Total_ce
    ENDIF ELSE IF loop eq 1 THEN BEGIN
      var1i = DOUBLE(v1i.Field16)
      var2i = DOUBLE(v2i.Field16)

      dn = oplus
      dnce = oplusce
    ENDIF ELSE IF loop eq 2 THEN BEGIN
      var1i = DOUBLE(v1i.Field20)
      var2i = DOUBLE(v2i.Field20)

      dn = odouble
      dnce = odoublece
    ENDIF ELSE IF loop eq 5 THEN BEGIN
      var1i = DOUBLE(v1i.Field24)
      var2i = DOUBLE(v2i.Field24)

      dn = strip
      dnce = stripce
    ENDIF ELSE IF loop eq 4 THEN BEGIN
      var1i = DOUBLE(v1i.Field28)
      var2i = DOUBLE(v2i.Field28)

      dn = sdouble
      dnce = sdoublece
    ENDIF ELSE IF loop eq 3 THEN BEGIN
      var1i = DOUBLE(v1i.Field32)
      var2i = DOUBLE(v2i.Field32)

      dn = splus
      dnce = splusce
    ENDIF ELSE IF loop eq 6 THEN BEGIN
      var1i = DOUBLE(v1i.Field36)
      var2i = DOUBLE(v2i.Field36)

      dn = H
      dnce = Hce
    ENDIF ELSE IF loop eq 7 THEN BEGIN
      var1i = DOUBLE(v1i.Field40)
      var2i = DOUBLE(v2i.Field40)

      dn = Na
      dnce = Nace
    ENDIF ELSE IF loop eq 8 THEN BEGIN
      var1i = DOUBLE(v1i.Field44)
      var2i = DOUBLE(v2i.Field44)

      dn = Hot
      dnce = Hotce
    endif




    Yran = [1d-2, 1d4]

    nne = [DOUBLE(v1i.Field52),DOUBLE(v2i.Field52)]
    nnn = [var1i,var2i]

    IF KEYWORD_SET(RELATIVE) THEN BEGIN
      IF species NE 'e-' THEN nnn = nnn/nne
    ENDIF

    rrr = [v1i.Field11, v2i.Field11]
    allind = WHERE((rrr GT 5.37) AND (rrr LT lims[1]))
    nnnall = nnn(allind)
    rrrall = rrr(allind)


    ind1 = where((rrr GT lims[0]) AND (rrr LT lims[1])) ; Changing this from 6 and 15.4 for this
    ;ind = where((rrr LE 6))
    rr1 = rrr(ind1)
    nn1 = nnn(ind1)

    spc_super = ['Ne_total','$O^+$','$O^{++}$','$S^{+++}$','$S^{++}$','$S^+$','$H^+$','$Na^+$','$O^+_{Hot}$']
    spc_lower = ['Ne_total','O+','O++','S+++','S++','S+','H+','Na+','O+_Hot']

    if loop eq 0 then begin
    ytck = ['$10^{-1}$','$10^{0}$','$10^{1}$','$10^{2}$','$10^{3}$','$10^{4}$']
    Yran = [1d-1, 1d4]
    endif
    if loop eq 1 then begin
      ytck = ['$10^{-2}$','$10^{-1}$','$10^{0}$','$10^{1}$','$10^{2}$','$10^{3}$']
      Yran = [1d-2, 1d3]
    endif
    if loop eq 2 then begin
      ytck = ['$10^{-2}$','$10^{-1}$','$10^{0}$','$10^{1}$','$10^{2}$']
      Yran = [1d-2, 1d2]
    endif
    if loop eq 3 then begin
      ytck = ['$10^{-2}$','$10^{-1}$','$10^{0}$','$10^{1}$','$10^{2}$','$10^{3}$','$10^{4}$']
      Yran = [1d-2, 1d4]
    endif
    if loop eq 4 then begin
      ytck = ['$10^{-2}$','$10^{-1}$','$10^{0}$','$10^{1}$','$10^{2}$','$10^{3}$']
      Yran = [1d-2, 1d3]
    endif
    if loop eq 5 then begin
      ytck = ['$10^{-2}$','$10^{-1}$','$10^{0}$','$10^{1}$','$10^{2}$']
      Yran = [1d-2, 1d2]
    endif
    if loop eq 6 then begin
      ytck = ['$10^{-2}$','$10^{-1}$','$10^{0}$','$10^{1}$','$10^{2}$']
      Yran = [1d-2, 1d2]
    endif
    if loop eq 7 then begin
      ytck = ['$10^{-2}$','$10^{-1}$','$10^{0}$','$10^{1}$','$10^{2}$','$10^{3}$']
      Yran = [1d-2, 1d3]
    endif
    if loop eq 8 then begin
      ytck = ['$10^{-2}$','$10^{-1}$','$10^{0}$','$10^{1}$','$10^{2}$','$10^{3}$']
      Yran = [1d-2, 1d3]
    endif
    
    
    
    spc_title = spc_super[loop]




    ;  /XLOG, /YLOG
    ; Plot Data
      dn = double(dn)
    p12  = plot(rho_ce[0:550], dn[0:550], color = 'black', Title = spc_title,sym_fill_color= 'black', XRANGE = XRANG, YRANGE = Yran, /XLOG, /YLOG, linestyle = '', symbol = '.', xtitle = '$\rho _c_e_q (R_J)$', $
      ytitle = '$N (cm^{-3})$',Dimensions=[1750,900],FONT_SIZE = 20,sym_size = 4, ytickname = ytck, XSTYLE = 1, /sym_filled)

    dn = double(dn)
    ;p13 = plot(rho_sc[0:550], dn[0:550], symbol = '.',linestyle = '',sym_size = 2, /overplot)
    dnce = double(dnce)
    p12 = plot(rho_ce[0:550], dnce[0:550], symbol = '.',sym_fill_color= 'blue', linestyle = '',sym_size = 4, color = 'blue', /overplot, /sym_filled)




    t1 = TEXT(0.53, 0.75,'$N( \rho ) = d a ( \rho/6 )^{b} , 6 R_J \leq  \rho  \leq  15.2 R_J$', /NORMAL, FONT_SIZE=16, FONT_NAME='Helvetica', COLOR = 'Black')


    IF LT6 EQ 1 THEN BEGIN
      p12.XTICKVALUES = [5,6]
      p12.XTICKNAME = ['5','6']
      p12.XMINOR = 0
    ENDIF ELSE BEGIN
      p12.XTICKVALUES = [5,6,7,8,9,10,20,30,40]
      p12.XTICKNAME = ['5','6','7','8','9','10','20','30','40']
      p12.XMINOR = 0
    ENDELSE


    ; Power law fit
    ; Remove zero points because of log spacing
    ind1n0 = WHERE(nn1 NE 0)
    nn1 = nn1(ind1n0)
    rr1 = rr1(ind1n0)
    par1 = linfit(alog(rr1),alog(nn1))


    ; debugging
    ;print, species, exp(pari[0]), pari[1]
    ;y    = exp(par1[0])*exp(par1[1]*alog(rr1)) +
    rrfull = [rr1]
    rrs = sort(rrfull)
    rrfull = rrfull(rrs)

    ; Physical constant we use to move fit up or down
    ;d = [1.2d,1.35d,1d,1d,1.3d,1.7d,1.15d,1.4d,1d,1d] ; If we want scaling .............
    d = [1.2d, 1.35d,1d,1d,1.3d,1.7d,1.15d,1.4d,1d] ; If we want scaling
    d = [1.2d, 1.35d,1d,1.5d,1.2d,1d,1.15d,1.4d,1d] ; If we want scaling
    ;d = [1d,1d,1d,1d,1d,1d,1d,1d,1d,1d]
    dspc = d[loop]
    dstr = STRING(FLOAT(dspc),FORMAT='(F5.2)')



    y1   = exp(par1[0])*rrfull^par1[1]
    y    = dspc[0]*y1[*] ; equivalent statement


    a1 = 1987
    b1 = -8.2
    a2 = 14
    b2 = -3.2
    a3 = 0.05
    b3 = -0.65

    overlap = 4601 ; Used manually for where curves overlap

    rrf = DINDGEN(17500)/500d + 6d ; Array to fill out plot
    nnf = a1*(rrf/6d)^(b1) + a2*(rrf/6d)^(b2) + a3*(rrf/6d)^(b3)
    IF loop eq 0 THEN p12 = plot(rrf,nnf,/OVERPLOT,color = 'purple') ; Only plot the 2011 line if it is total charge density being plotted

    ; Empirical - Scale Fran's code - Found out 4601 for rrf =~ max of rri
    rrf2 = DINDGEN(12900)/500d + 15.202 ; Patch together other solution
    nnft = a1*(rrf2/6d)^(b1) + a2*(rrf2/6d)^(b2) + a3*(rrf2/6d)^(b3)
    nnf2 = nnft*min(y)/nnf(overlap)

    p12 = plot(rrfull,y,/OVERPLOT,color='black')
    
    scl_color = 'deep sky blue'
    fr_color  = 'dark violet'
    ; Fran's paper 2011 Bagenal/Del
    a1 = 1987
    b1 = -8.2
    a2 = 14
    b2 = -3.2
    a3 = 0.05
    b3 = -0.65

    overlap = 4601 ; Used manually for where curves overlap



    ; Empirical - Scale Fran's code - Found out 4601 for rrf =~ max of rri
    rrf2 = DINDGEN(12900)/500d + 15.202 ; Patch together other solution
    nnft = a1*(rrf2/6d)^(b1) + a2*(rrf2/6d)^(b2) + a3*(rrf2/6d)^(b3)
    nnf2 = nnft*min(y)/nnf(overlap)
    p12 = plot(rrf2,nnf2,/OVERPLOT,color = scl_color)

    stra_1 = string(ROUND(FLOAT(exp(par1[0])*6d^(par1[1])))) ; No decimal places
    strb_1 = string(par1[1],FORMAT = '(F5.2)') ; 2 decimal places

    ;print, species
    print, exp(par1[0])*6d^(par1[1])
    print, par1[1]


    SUM_Dens = 0
    IF SUM_Dens EQ 1 THEN BEGIN
      rrf6 = rrfull/6d
      n1 = 48.0527*rrf6^(-5.31)
      n2 = 71.3144*1.4*rrf6^(-6.87)
      n3 = 132.802*rrf6^(-4.61)
      n4 = 447.370*1.35*rrf6^(-7.46)
      n5 = 87.0067*rrf6^(-6.96)
      n6 = 80.1781*rrf6^(-6.56)
      n7 = 373.041*1.3*rrf6^(-6.58)
      ntot = n1*1d + n2*1d + n3*1d + n4*1d + n5*2d + n6*1d + n7*2d + n8*3d

      p12 = plot(rrfull,ntot,/OVERPLOT,color='green')



    ENDIF

    p12.XTICKVALUES = [5,6,7,8,9,10,20,30, 40]
    p12.XTICKNAME = ['5','6','7','8','9','10','20','30', '40']
    p12.XMINOR = 0

    ;str3 = STRCOMPRESS(STRING('a_1 = ', str1, ', b_1 = ', str2, ', a_2 = ', str4, FORMAT='(A,A,A,A,A,A)'))
    ;t2 = TEXT(0.61, 0.75,str3, /NORMAL, FONT_SIZE=16, FONT_NAME='Helvetica', COLOR = 'Black')


    print, min(y)/nnf(overlap)

 if loop eq 0 then t2   = TEXT(0.53,0.8,'$N_{2011}( \rho ) = 1987 ( \rho/6 )^{-8.2} + 14 ( \rho/6 )^{-3.2} + 0.05 ( \rho/6 )^{-0.65}, \rho  \geq  6 R_J$', color = fr_color, FONT_SIZE=16, FONT_NAME='Helvetica')
    str4 = string(min(y)/nnf(overlap),FORMAT = '(F5.2)') ; 2 decimal places
    str4 = string(min(y)/nnf(overlap),FORMAT = '(F5.3)') ; Using this for now
    t3   = TEXT(0.53,0.7,'$n( \rho ) = c N_{2011}, \rho  > 15.2 R_J$', COLOR = scl_color, FONT_SIZE=16, FONT_NAME='Helvetica')
    str3 = STRCOMPRESS(STRING('$a = $', stra_1, '$, b = $', strb_1, '$, c = $', str4, '$, d = $', dstr, FORMAT='(A,A,A,A,A,A,A,A)'))
      t4   = TEXT(0.53, 0.65,str3, /NORMAL, FONT_SIZE=16, FONT_NAME='Helvetica', COLOR = 'Black')

    ; Determine densities of cold blobs - Only going to include in the total charge density for now for ease:
    ; Using a manual system here - It's kind of clunky
    cb1_csv = read_csv('V1_JUP_M_Dec20.csv')
    indices = [3:772] ; Fill this with correct values
    ncb1    = MAKE_ARRAY(N_ELEMENTS(indices))
    rcb1    = v1i.Field08[indices] ; Should have the same values

    Densities = [[cb1_csv.Field31],[cb1_csv.Field36],[cb1_csv.Field41],[cb1_csv.Field46],[cb1_csv.Field51],[cb1_csv.Field56],[cb1_csv.Field61],[cb1_csv.Field66],[cb1_csv.Field71]]
    Arguments = [[cb1_csv.Field28],[cb1_csv.Field33],[cb1_csv.Field38],[cb1_csv.Field43],[cb1_csv.Field48],[cb1_csv.Field53],[cb1_csv.Field58],[cb1_csv.Field63],[cb1_csv.Field68]]
    Charges   = [[cb1_csv.Field30],[cb1_csv.Field35],[cb1_csv.Field40],[cb1_csv.Field45],[cb1_csv.Field50],[cb1_csv.Field55],[cb1_csv.Field60],[cb1_csv.Field65],[cb1_csv.Field70]]
    Dens_new  = Densities

    FOR j = 0, N_ELEMENTS(Densities[*,0]) - 1 DO BEGIN
      FOR i = 0, N_ELEMENTS(Densities[0,*]) - 1 DO BEGIN
        IF i eq 0 THEN CONTINUE
        IF Arguments(j,i) EQ 4 OR Arguments(j,i) EQ 5 THEN BEGIN
          Dens_new(j,i) = Densities[j,i]*Densities[j,0]
        ENDIF
      ENDFOR
    ENDFOR

    strng = spc_lower[loop] + 'density_power_law.png'
    p12.SAVE, strng, RESOLUTION = 600

  endfor ; exit loop loop

End